/* Time-stamp: "error.c, Wednesday October 1 1997 22:9:25CDT by rjonkman@ittc.ukans.edu on pepe.ittc.ukans.edu" */

/*
 * proj : NetPipe
 * file : error.c
 * prog : Roel Jonkman
 * date : October 10th 1995
 * rev  : 3
 * desc : Error & debug message handling routines ripped from NetSpec
 * rcs  : $Id$
 *
 * This software is copyrighted see COPYRIGHTS
 *
 * $Log$
 */



/*
 * system includes
 */

#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>




/*
 * local includes
 */

#include "error.h"



#define ERRORSIZE 2048



static char errorMessages[ERRORSIZE];
static char *ptr2Errors = errorMessages;
static FILE *where = stderr;

int debugLvl;



void debugMsg (const int debugLevel, const char *file, const int line, const char *function, 
	       const char *call, const char *what) {
  /* input   : debugLevel, filename, lineno, function, call, message.
   * returns : -
   * output  : -
   * desc    : generic debug reporting routine.
   */

  char hostname[64];

  if (debugLevel > debugLvl)
    return;

  gethostname (hostname, 63);
  sprintf (ptr2Errors, "%s:%d => Debug : %s\n", hostname, (int)getpid(),
	   what);
  ptr2Errors += strlen (ptr2Errors);

  sprintf (ptr2Errors, "EDI => %s:%d, %s, %s\n",file, line, function, call);
  ptr2Errors += strlen (ptr2Errors);

  dumpError(File);
  
  return;

}



void errorMsg (const char *file, const int line, const char *function, 
	       const char *call, const char *why) {
  /* input   : filename, lineno, function, call, user message.
   * returns : -
   * output  : -
   * desc    : generic error reporting routine.
   */

  char hostname[64];

  gethostname (hostname, 63);
  sprintf (ptr2Errors, "%s:%d => Error : %s\n", hostname, (int)getpid(),
	   why);
  ptr2Errors += strlen (ptr2Errors);

  if (debugLvl > 0) {

    sprintf (ptr2Errors, "EDI => %s:%d, %s, %s\n",file, line, function, 
	     call);
    ptr2Errors += strlen (ptr2Errors);

  }

  return;

}



void dumpError (enum where_t where2) { 
  /* input   : -
   * returns : -
   * output  : -
   * desc    : dumps error to a file or a buffer or both.
   */

  switch (where2){

  case Buffer:
    break;

  case File:
    fprintf (where, "%s", errorMessages);
    fflush (where);
    ptr2Errors = errorMessages;
    errorMessages[0] = 0x0;
    break;

  case Both:
    fprintf (where, "%s", errorMessages);
    fflush (where);
    break;

  }

}



void clearError (void){
  /* input   : -
   * returns : -
   * output  : -
   * desc    : clears the errorBuffer.
   */

  ptr2Errors = errorMessages;
  errorMessages[0] = 0;

}



char *getError(void){
  /* input   : -
   * returns : a ptr to the errorBuffer
   * output  : -
   * desc    : returns a pointer to the error buffer.
   */

  return (errorMessages);

}



int openError (const char *aFileName) {
  /* input   : filename where to write the errorfile to.
   *           the filename may not contain characters like ~
   * returns : NOK if file could not be opened, OK if no errors occured.
   * output  : -
   * desc    : opens up an error file
   */


  if (!strncmp (aFileName, "stdout", 7)) {
    
    where = stdout;
    return OK;
	
  }

  if (!strncmp (aFileName, "stderr", 7)) {

    where = stderr;
    return OK;

  }

  if ((where = fopen (aFileName, "w")) != (FILE*)NULL)
    return OK;

  return NOK;

}



void closeError (void){
  /* input   : -
   * returns : -
   * output  : -
   * desc    : closes errorfile.
   */

  fclose (where);

}
