/* Time-stamp: "error.h, Wednesday October 1 1997 22:10:16CDT by rjonkman@ittc.ukans.edu on pepe.ittc.ukans.edu" */

/*
 * proj : NetPipe
 * file : error.h
 * prog : Roel Jonkman
 * date : October 10th 1995
 * rev  : 3
 * desc : Error & Debug message handling routines, reaped from NetSpec
 * rcs  : $Id: error.h,v 1.1 1995/06/28 22:40:37 rjonkman Exp rjonkman $
 *
 * This software is copyrighted see COPYRIGHTS
 *
 * $Log$
 */

#ifndef ERROR_H
#define ERROR_H

#include <errno.h>
#include <string.h>

#include "gendef.h"


#ifndef DEBUGLEVEL
#define DEBUGLEVEL 1024
#endif

typedef enum where_t {

  Buffer = 0,
  File = 1,
  Both = 2

} where_t;

/*
 * global variables,
 */


extern int debugLvl;



/*
 * sunOs problemo
 */

#ifdef sunos
#include <errno.h>
extern char *sys_errlist[];
#define strerror(no)(sys_errlist[no])
#endif



/*
 * shortcuts for the error routines
 */

#ifdef __GNUC__
#define FUNCTIONNAME __FUNCTION__
#else
#define FUNCTIONNAME "no function name available"
#endif

#ifdef DEBUG
#define debug(call, what) debugMsg(DEBUGLEVEL, __FILE__, __LINE__,\
				   FUNCTIONNAME, call, what)
/* input   : call, message
 * returns : -
 * output  : -
 * desc    : shortcut for debugMsg, DEBUGLEVEL should be defined
 *           at the beginning of each module to indicate what
 *           level of debugging is necc to have this reported.
 */
#else
#define debug(call, what)
/* input   : call, message
 * returns : -
 * output  : -
 * desc    : bogus debug call, empty, only if you compile with DEBUG
 */
#endif


#define error(call, why) errorMsg(__FILE__, __LINE__, FUNCTIONNAME,\
				  call, why)
/* input   : call, message
 * returns : -
 * output  : -
 * desc    : shortcut for errorMsg
 */

#define systemError(call) errorMsg(__FILE__, __LINE__, FUNCTIONNAME, call,\
				   strerror(errno))
/* input   : systemcall
 * returns : -
 * output  : -
 * desc    : shortcut for errorMsg, why == strerror(errno)
 */

void debugMsg (const int debugLevel, const char *file, const int line, const char *function, 
	       const char *call, const char *what) ;
  /* input   : debugLevel, filename, lineno, function, call, message.
   * returns : -
   * output  : -
   * desc    : generic debug reporting routine.
   */

void errorMsg (const char *file, const int line, const char *function, 
	       const char *call, const char *why) ;
  /* input   : filename, lineno, function, call, user message.
   * returns : -
   * output  : -
   * desc    : generic error reporting routine.
   */

void dumpError (enum where_t) ; 
  /* input   : -
   * returns : -
   * output  : -
   * desc    : dumps error to a file or a buffer or both.
   */

void clearError (void);
  /* input   : -
   * returns : -
   * output  : -
   * desc    : clears the errorBuffer.
   */

char *getError(void);
  /* input   : -
   * returns : a ptr to the errorBuffer
   * output  : -
   * desc    : returns a pointer to the error buffer.
   */

int openError (const char *aFileName) ;
  /* input   : filename where to write the errorfile to.
   *           the filename may not contain characters like ~
   * returns : NOK if file could not be opened, OK if no errors occured.
   * output  : -
   * desc    : opens up an error file
   */

void closeError (void);
  /* input   : -
   * returns : -
   * output  : -
   * desc    : closes errorfile.
   */

#endif


