/* Time-stamp: "netpipe.c, Saturday June 20 1998 16:0:25CDT by rjonkman@ittc.ukans.edu on pepe.ittc.ukans.edu" */

/*
 * proj : NetPipe
 * file : netpipe.c
 * prog : Roel Jonkman
 * date : September 30th 1997
 * rev  : 1
 * desc : Network pipe, heavily inspired by netcat and ttcp, reaped for
 *        the most part from NetSpec.
 * rcs  : $Id$
 *
 * This software is copyrighted see COPYRIGHTS
 * 
 * $Logs$
 */


/*
 * system includes
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include "error.h"
#include "buffer.h"
#include "ipaddress.h"
#include "sockets.h"
#include "peek.h"
#include "process.h"

#if !defined INADDR_LOOPBACK
#define INADDR_LOOPBACK         (unsigned int)0x7F000001
#endif

#define DEFBUFSIZE 32768
#define DEFPORT 5001

/*
 * typedefs
 */

typedef void  (*signalFunction)(int aSignal);

/*
 * internal prototypes
 */

static void usage       (void);
static void about       (void);
static void bailOut     (int signal);
static void rollOver    (void);
static int connectPeer(unsigned int ownAddress, unsigned short ownPort, 
		       unsigned int peerAddress, unsigned short peerPort, 
		       int window, int tcpmtu);
static int readWrite(int input, int output, int pipe, int chunkSize);

static char *realName;
static int verbose = 0;


int main(int argc, char *argv[]) {

  int errorFlag = FALSE;	
  int aFlag;
  int bufSize = DEFBUFSIZE;
  extern char *optarg;
  extern int optind;
  int window = 65536; /* f*ck standard windowz, too damn small for real thruput */
  int input = 0; /* stdin */
  int output = 1; /* stdout */
  int pipe; /* socket */
  int tcpmtu = -1;
  unsigned int peerAddress = INADDR_LOOPBACK;
  unsigned short peerPort = DEFPORT;
  unsigned int ownAddress = INADDR_ANY;
  unsigned short ownPort = DEFPORT;

  realName = argv[0];

  while (!errorFlag && (aFlag = getopt (argc, argv, "vwhm:b:l:o:i:p:s:x:y:")) != -1)
    switch (aFlag){
      
    case 'b': 
      window = atoi (optarg);
      break;
      
    case 'l': 
      bufSize = atoi (optarg);
      break;

    case 'o':
      if ((output = open(optarg, O_WRONLY | O_TRUNC, 0644)) == -1){

	 systemError("open");
	 dumpError(File);
	 exit(1);

      }
      break;
      
    case 'i':      
      if ((input = open(optarg, O_RDONLY)) == -1){

	 systemError("open");
	 dumpError(File);
	 exit (1);

      }
      lseek (input, 0, SEEK_SET); /* ignore, since in some case you can't. */
      break;
      
    case 'p':      
      peerPort = atoi (optarg);
      break;
      
    case 's':
      if ((peerAddress = str2IpAddr (optarg)) == 0){

        dumpError(File);
        return 1;

      }
      break;

    case 'x':      
      ownPort = atoi (optarg);
      break;
      
    case 'y':
      if ((ownAddress = str2IpAddr (optarg)) == 0){

        dumpError(File);
        return 1;

      }
      break;

    case 'v':
      verbose = 1;
      break;

    case 'm':
      tcpmtu = atoi(optarg);
      break;

    case 'h':
      usage();
      exit(0);
      break;

    case 'w':
      about();
      exit(0);
      break;

    default:
      errorFlag = 1;
      
    }
  
  if (errorFlag) {

    usage();
    exit(1);

  }


#ifdef sunos
  on_exit (rollOver, 1);
#else
  atexit (rollOver);
#endif

  signal (SIGHUP, bailOut);
  signal (SIGINT, bailOut);
  signal (SIGQUIT, bailOut);
  signal (SIGILL, bailOut);
  signal (SIGTRAP, bailOut);
  signal (SIGABRT, bailOut);
  signal (SIGFPE, bailOut);
  signal (SIGBUS, bailOut);
  signal (SIGSEGV, bailOut);
  signal (SIGTERM, bailOut);
  signal (SIGURG, SIG_IGN);
  signal (SIGUSR1, SIG_IGN);
  signal (SIGUSR2, SIG_IGN);

  if ((pipe = connectPeer (ownAddress, ownPort, peerAddress, peerPort, 
			   window, tcpmtu)) == -1){
    
    dumpError(File);
    exit(1);

  }

  if (setvbuf (stdin, (char*)NULL, _IONBF, 0) != 0)
    fprintf (stderr, "Warning: could not set buffering of stdin to 0i, may result in loss of data.\n");

  if (setvbuf (stdout, (char*)NULL, _IONBF, 0) != 0)
    fprintf (stderr, "Warning: could not set buffering of stdout to 0, may result in loss of data.\n");

  if (readWrite (input, output, pipe, bufSize) == NOK){

    dumpError(File);
    exit(1);

  }

  shutdown (pipe,2);
  close (pipe);

  return 0;
  
}



static int connectPeer(unsigned int ownAddress, unsigned short ownPort,
		       unsigned int peerAddress, unsigned short peerPort,
		       int window, int tcpmtu){
  /* 
   * input   : peer name and port.
   * returns : -
   * output  : -
   * desc    : connect to host, first pursue a connect, if connect 
   *           results in a ECONNREFUSED, revert to accept, assuming that
   *           the other end has not started yet. Does fancy namelookup
   *           garbage too.
   */

  int pipe;
  extern int errno;
  unsigned short tempPort = ownPort;
  unsigned int tempAddress = ownAddress;
  

  if (createSocket(&pipe, AF_INET, SOCK_STREAM) == NOK){
    
    dumpError(File);
    return -1;

  }

  if (bsdRcvBuf (pipe, SET, &window) == NOK)
    fprintf (stderr, "Failed to set receive buffer, reverted to %d bytes\n", window);
  if (bsdXmtBuf (pipe, SET, &window) == NOK)
    fprintf (stderr, "Failed to set transmit buffer, reverted to %d bytes\n", window);
  if (tcpmtu > 32 && bsdTcpMaxSegmentSize (pipe, SET, &tcpmtu) == NOK)
    fprintf (stderr, "Failed to set tcp mtu, reverted to %d bytes\n", tcpmtu);
   
  /* plain connect so we want the os to pick a port. */

  if (ownPort == DEFPORT)
    ownPort=0;
 
  if (bindSocket(pipe, AF_INET, &ownAddress, &ownPort) == NOK){
    
    dumpError(File);
    return -1;
    
  }

  if (connectSocket(pipe, AF_INET, peerAddress, peerPort) == NOK) {

    if (errno == ECONNREFUSED){

      close (pipe);

      if (createSocket(&pipe, AF_INET, SOCK_STREAM) == NOK){
	
	dumpError(File);
	return -1;
	
      }
      
      if (bsdRcvBuf (pipe, SET, &window) == NOK)
	fprintf (stderr, "Failed to set receive buffer, reverted to %d bytes\n", window);
      if (bsdXmtBuf (pipe, SET, &window) == NOK)
	fprintf (stderr, "Failed to set transmit buffer, reverted to %d bytes\n", window);
      if (tcpmtu > 32 && bsdTcpMaxSegmentSize (pipe, SET, &tcpmtu) == NOK)
	fprintf (stderr, "Failed to set tcp mtu, reverted to %d bytes\n", tcpmtu);
      
      ownAddress = tempAddress;
      ownPort = tempPort;
    
      if (bindSocket(pipe, AF_INET, &ownAddress, &ownPort) == NOK){
	
	dumpError(File);
	return -1;
	
      }
      
      if (setupSocketQueue(pipe, 1) == NOK) {
	
	dumpError(File);
	return -1;

      }
	
      if (wait4Request (&pipe, FALSE) == NOK) {
	
	dumpError(File);
	return -1;

      }

    } else {
      
      dumpError(File);
      return -1;

    }

  }
	
  return pipe;

}



static int readWrite(int input, int output, int pipe, int chunkSize){
  /* 
   * input   : input, output and pipe, and max chunksize.
   * returns : nothing exits on failure and end.
   * output  : -
   * desc    : loops till an eof occurs on either input or pipe.
   * note    : stole name from netcat, readwrite in netcat is same thing.
   */

  void *ptr2InputBuffer;
  void *ptr2InputBufferBase;
  long inputCount=0, outputCount=0, pipeInCount=0, pipeOutCount=0;
  int readBytes, writeBytes, i;
  struct peekfd peekStuff[2];

  if ((ptr2InputBuffer = allocateBuffer(getpagesize(), 0, chunkSize,
					&ptr2InputBufferBase)) == NULL){
    
    dumpError(File);
    return NOK;

  }

  memset ((void*)peekStuff, 0x0, 2*sizeof(struct peekfd));

  peekStuff[0].fd = input;
  peekStuff[0].events |= PEEKIN | PEEKRDNORM;
  peekStuff[1].fd = pipe;
  peekStuff[1].events |= PEEKIN | PEEKRDNORM | PEEKHUP;
  
  startProcessStats();

  while (peek(peekStuff, 2, -1) != -1){

    if (peekStuff[1].revents & PEEKHUP)
      break;


    if (peekStuff[0].revents & PEEKIN || 
	peekStuff[0].revents & PEEKRDNORM){
      
      if ((readBytes = read (input, ptr2InputBuffer, chunkSize)) == -1)
	dumpError(File);

      if (readBytes == 0)
	break;
      
      if ((writeBytes = write (pipe, ptr2InputBuffer, readBytes)) == -1)
	  dumpError(File);

      pipeOutCount += writeBytes;
      inputCount += readBytes;

    }
      
    if (peekStuff[1].revents & PEEKIN || 
	peekStuff[1].revents & PEEKRDNORM){

      if ((readBytes = read (pipe,ptr2InputBuffer, chunkSize)) == -1)
	dumpError(File);
      
      if (readBytes == 0)
	break;
      
      if ((writeBytes = write (output, ptr2InputBuffer, readBytes)) == -1)
	  dumpError(File);

      pipeInCount += readBytes;
      outputCount += writeBytes;

    }

  }
 
  /* Make sure we don't got anything left. */

  for (i=0; i < 100; i++){

    





  }






  stopProcessStats();
  fputs ("\n\nNetPipe Transfer statistics \n\n", stderr);
  
  fprintf (stderr, "  Total time consumed\t\t: %15.2f s\n", getDuration());
  fprintf (stderr, "  Bytes received\t\t: %15ld bytes\n", pipeInCount);
  fprintf (stderr, "  Thruput received\t\t: %15.3f Mbps\n", 
	   (double)(pipeInCount*8) / getDuration()/1e6);
  fprintf (stderr, "  Bytes transmitted\t\t: %15ld bytes\n", pipeOutCount);
  fprintf (stderr, "  Thruput transmitted\t\t: %15.3f Mbps\n", 
	   (double)(pipeOutCount*8) / getDuration()/1e6);

  if (verbose)
    dumpProcessStats(stderr);

  free (ptr2InputBufferBase);

  return OK;

}


static void rollOver(void) {
  /* 
   * input   : signal
   * returns : -
   * output  : -
   * desc    : Gracefull exit procedure in case of an emergency condition.
   */
  
  
}



static void bailOut (int signal) {
 /* 
  * input   : signal
  * returns : -
  * output  : -
  * desc    : Causes exit to be called which calls rollOver.
  *           Somewhat fudged, but works oke.
  */

  exit(1);

}



static void usage (void) {
 /* 
  * input   : -
  * returns : -
  * output  : -
  * desc    : tell a user, how to use this wonderfull tool.
  */

  fprintf (stderr, "\nNetPipe, V1.0a by Roelof JT Jonkman\n\n");
  fprintf (stderr, "Usage: %s [options]\n\n", realName);
  fprintf (stderr, "Options:\n\n"); 
  fprintf (stderr, "-b buffer \t\tsocketbuffer(upper bound on tcp window)\n");
  fprintf (stderr, "-l length \t\tlength\n");
  fprintf (stderr, "-o filename \t\tfile to write to (default stdout)\n");
  fprintf (stderr, "-i filename \t\tfile to read from (default stdin)\n");
  fprintf (stderr, "-p port \t\tport to connect to (default 5001, same as ttcp)\n");
  fprintf (stderr, "-s address \t\tto connect to (default localhost)\n");
  fprintf (stderr, "-h \t\t\tthis cruft\n");
  fprintf (stderr, "-w \t\t\twho did this, and where to get more.\n");
  fprintf (stderr, "\nObscure options:\n\n"); 
  fprintf (stderr, "-v \t\t\tVerbose statistics, not for the faint of heart\n");
  fprintf (stderr, "-m size\t\t\tset tcp segmentation size, 32 < mtu < ifmtu\n");
  fprintf (stderr, "-x port\t\t\tlocal port to bind on\n");
  fprintf (stderr, "-y address\t\tlocal address to use (Multihoming)\n");
  
  return;

}



static void about (void) {
 /* 
  * input   : -
  * returns : -
  * output  : -
  * desc    : tell the user some good things about NetPipe.
  */

  fprintf (stderr, "\nNetPipe C '97 Roelof JT Jonkman\n");
  fprintf (stderr, "Inspired by netcat and ttcp\n");
  fprintf (stderr, "Permission to use modify etc, as long as this is included.\n");
  fprintf (stderr, "NO WARRANTY WHATSOEVER\n\n");
  fprintf (stderr, "More information and latest incarnations:\n\n");
  fprintf (stderr, "FTP  ftp://ftp.ittc.ukans.edu/pub/src/netpipe\n");
  fprintf (stderr, "HTTP http://www.ittc.ukans.edu/~rjonkman/netpipe\n");
  fprintf (stderr, "MAIL rjonkman@ittc.ukans.edu\n\n");
  return;

}
