/* Time-stamp: "peek.c, Wednesday November 5 1997 15:37:56CST by rjonkman@ittc.ukans.edu on pepe.ittc.ukans.edu" */

/*
 * proj : NetPipe
 * file : peek.c
 * prog : Roel Jonkman
 * date : December 20st 1996
 * rev  : 1
 * desc : Wrapper around the poll and select saga such that this 
 *        is transparant. (Linux where the hell is poll??? I know it 
 *        SVR4 but still, any other os has it....)
 *        reaped from NetSpec
 * rcs  : $Id$
 *
 * This software is copyrighted see COPYRIGHTS
 *
 * $Log$
 */

#include "peek.h"

#if defined linux || defined freebsd

#include <sys/types.h>
#include <sys/time.h>

#define DEBUGLEVEL 5

#include "error.h"


int peek(struct peekfd ptr2PeekFileDescriptors[], int peekEntries, 
	 int timeOut) {
  /*
   * input   : see man poll on system with poll, substitute poll with
   *           peek when reading.
   * returns : dito.
   * output  : dito.
   * desc    : Fakes poll by using select, obviously will not 
   *           support all the crap that poll does.
   *           Supported options:
   *           PEEKIN, PEEKOUT
   */

  struct timeval tv;
  fd_set rdfs;
  fd_set wrfs;
  int i;
  int retval;
  int max = 0;

  debug ("peek", "begin of function");

  tv.tv_sec = timeOut / 1000;
  tv.tv_usec = 0 % 1000;
  
  FD_ZERO(&rdfs);
  FD_ZERO(&wrfs);

  for (i=0; i<peekEntries;i++) { /* set events */
    
    if (ptr2PeekFileDescriptors[i].events & PEEKIN)
      FD_SET(ptr2PeekFileDescriptors[i].fd, &rdfs);
    else if (ptr2PeekFileDescriptors[i].events & PEEKOUT)
      FD_SET(ptr2PeekFileDescriptors[i].fd, &wrfs);

    if (ptr2PeekFileDescriptors[i].fd > max)
      max = ptr2PeekFileDescriptors[i].fd;
    
  }
  
  if (timeOut < 0)
    retval = select(max+1,&rdfs, &wrfs, NULL, NULL);
  else
    retval = select(max+1,&rdfs, &wrfs, NULL, &tv);
 
  if (retval<0)
    return retval;

  else {
    
    for (i=0; i<peekEntries;i++) { /* get events */

      if (FD_ISSET(ptr2PeekFileDescriptors[i].fd, &rdfs))
	ptr2PeekFileDescriptors[i].revents = PEEKIN;
      else if (FD_ISSET(ptr2PeekFileDescriptors[i].fd, &wrfs))
	ptr2PeekFileDescriptors[i].revents = PEEKOUT;

    }
    
    return retval;

  }

}

#endif
