/* Time-stamp: "peek.h, Friday April 25 1997 20:1:24CDT by rjonkman@ittc.ukans.edu on pepe.tisl.ukans.edu" */

/*
 * proj : NetSpec
 * file : peek.h
 * prog : Roel Jonkman
 * date : December 20st 1996
 * rev  : 1
 * desc : Wrapper around the poll and select saga such that this 
 *        is transparant. (Linux where the hell is poll??? I know it 
 *        SVR4 but still, any other os has it....)
 * rcs  : $Id$
 *
 * This software is copyrighted see COPYRIGHTS
 *
 * $Log$
 */

#ifndef PEEK_H
#define PEEK_H

#if defined linux || defined freebsd

/*
 * So we make a peek call, that looks like poll, so we 
 * can macrofy peek if poll exists.
 */

#define	PEEKNORM	01	/* message available on read queue */
#define	PEEKPRI		02	/* priority message available */
#define	PEEKOUT		04      /* stream is writable */
#define	PEEKERR		010	/* error message has arrived */
#define	PEEKHUP		020	/* hangup has occurred */
#define	PEEKNVAL	040	/* invalid fd */
#define	PEEKRDNORM	0100	/* A non-priority message is available */
#define	PEEKRDBAND	0200	/* A band>0 message is available */
#define	PEEKWRNORM	0400    /* Same as PEEKOUT */
#define	PEEKWRBAND	01000	/* A priority band exists and is writable */
#define	PEEKMSG		02000	/* A SIGPEEK message has arrived */
#define	PEEKIN		PEEKNORM

struct peekfd {
	int	fd;
	int	events;
	int	revents;
};

int peek(struct peekfd[], int, int );

#else /* NOPOLL */



/*
 * Take, care in case poll is there, peek translates verbatim into poll,
 * it looks really cute, but is fairly devious. If the equivalent define
 * exists on the OS, the peek define will be there, otherwise not.
 */

#include <sys/poll.h>

#ifdef POLLNORM
#define	PEEKNORM	POLLNORM
#endif

#ifdef POLLPRI
#define	PEEKPRI		POLLPRI
#endif

#ifdef POLLOUT
#define	PEEKOUT		POLLOUT
#endif

#ifdef POLLERR
#define	PEEKERR		POLLERR
#endif

#ifdef POLLHUP
#define	PEEKHUP		POLLHUP
#endif

#ifdef POLLNVAL
#define	PEEKNVAL	POLLNVAL
#endif

#ifdef POLLRDNORM
#define	PEEKRDNORM	POLLRDNORM
#endif

#ifdef POLLRDBAND
#define	PEEKRDBAND	POLLRDBAND
#endif

#ifdef POLLWRNORM
#define	PEEKWRNORM	POLLWRNORM
#endif

#ifdef POLLWRBAND
#define	PEEKWRBAND      POLLWRBAND
#endif

#ifdef POLLMSG
#define	PEEKMSG		POLLMSG
#endif

#ifdef POLLIN
#define	PEEKIN		POLLIN
#endif

/*
 * This redefines the references to peek and peekfd.
 * the peek stuff may compiled in, but if this header is included 
 * before referencing it, it will never be called.
 */

#define peek(a, b, c) poll(a, b, c)

#define peekfd pollfd

#endif /* NOPOLL */

#endif
