/* Time-stamp: "process.c, Wednesday October 1 1997 22:13:4CDT by rjonkman@ittc.ukans.edu on pepe.ittc.ukans.edu" */

/*
 * proj : NetPipe
 * file : process.c
 * prog : Roelof Jonkman
 * date : June 27 1995
 * rev  : 1
 * desc : timestamping facility, is the same for every API, it's not API specific.
 * rcs  : $Id: stamp.c,v 1.1 1995/06/28 22:40:37 rjonkman Exp rjonkman $
 *
 * This software is copyrighted see COPYRIGHTS
 *
 * $Log$
 */



/*
 * includes
 */
#include <sys/time.h>
#include <sys/types.h>
#include <sys/resource.h>


#include "process.h"



typedef struct processStats {

  struct timeval begin;
  struct timeval end;

  struct timeval userCpu;
  struct timeval systemCpu;

  long pageReclaims;
  long pageFaults;
  long swaps;
  long inputBlocks;
  long outputBlocks;
  long inputMessages;
  long outputMessages;
  long signals;
  long voluntaryContextSwitches;
  long forcedContextSwitches;

} processStats;

typedef processStats *ptr2ProcessStats;



/*
 * local global stuff.
 */

static struct rusage rub, rue;
static struct timeval tvb, tve;
static struct processStats ps;



void calculateProcessStats (void){
  /* 
   * input   : 
   * returns :
   * output  :
   * desc    :
   */
 
  ps.begin.tv_sec = tvb.tv_sec;
  ps.begin.tv_usec = tvb.tv_usec;
  ps.end.tv_sec = tve.tv_sec;
  ps.end.tv_usec = tve.tv_usec;
  ps.userCpu.tv_sec = rue.ru_utime.tv_sec - rub.ru_utime.tv_sec;
  ps.userCpu.tv_usec = rue.ru_utime.tv_usec - rub.ru_utime.tv_usec;
  if (ps.userCpu.tv_usec < 0) {

    ps.userCpu.tv_sec--;
    ps.userCpu.tv_usec += 1e6;

  }
  ps.systemCpu.tv_sec = rue.ru_stime.tv_sec - rub.ru_stime.tv_sec;
  ps.systemCpu.tv_usec = rue.ru_stime.tv_usec - rub.ru_stime.tv_usec;
  if (ps.systemCpu.tv_usec < 0) {

    ps.systemCpu.tv_sec--;
    ps.systemCpu.tv_usec += 1e6;

  }
  ps.pageReclaims = rue.ru_minflt - rub.ru_minflt;
  ps.pageFaults = rue.ru_majflt - rub.ru_majflt;
  ps.swaps = rue.ru_nswap - rub.ru_nswap;
  ps.inputBlocks = rue.ru_inblock - rub.ru_inblock;
  ps.outputBlocks = rue.ru_oublock - rub.ru_oublock;
  ps.inputMessages = rue.ru_msgrcv - rub.ru_msgrcv;
  ps.outputMessages = rue.ru_msgsnd - rub.ru_msgsnd;
  ps.signals = rue.ru_nsignals - rub.ru_nsignals;
  ps.voluntaryContextSwitches = rue.ru_nvcsw - rub.ru_nvcsw;
  ps.forcedContextSwitches = rue.ru_nivcsw - rub.ru_nivcsw;

}



double getDuration (void){
  /* 
   * input   : 
   * returns :
   * output  :
   * desc    :
   */

  return (((double)tve.tv_sec + (double)tve.tv_usec/1e6)  -
    ((double)tvb.tv_sec + (double)tvb.tv_usec/1e6));

}



char *getStartTime (void){
  /* 
   * input   : 
   * returns :
   * output  :
   * desc    :
   */

  static char date[80];

  strftime (date, (size_t)80,"%m/%d/%y %H:%M:%S %Z",
            localtime ((time_t*)&tvb.tv_sec));

  return (date);

}


char *getEndTime (void){
  /* 
   * input   : 
   * returns :
   * output  :
   * desc    :
   */

  static char date[80];

  strftime (date, (size_t)80,"%m/%d/%y %H:%M:%S %Z",
            localtime ((time_t*)&tve.tv_sec));

  return (date);

}



void dumpProcessStats(FILE *where){
  /* 
   * input   : 
   * returns :
   * output  :
   * desc    :
   */

  double user,system;

  calculateProcessStats();

  system = 1e6 * ps.systemCpu.tv_sec + ps.systemCpu.tv_usec;
  user = 1e6 * ps.userCpu.tv_sec + ps.userCpu.tv_usec;

  fprintf (where, "\nProcess stats (rusage)\n\n");
  fprintf (where, "  System CPU time            \t: %15.0f us\n", system);
  fprintf (where, "  User CPU time              \t: %15.0f us\n", user);
  fprintf (where, "  Total CPU time             \t: %15.0f us\n", system + user);
  fprintf (where, "  Page faults                \t: %15ld\n", ps.pageFaults);
  fprintf (where, "  Page reclaims              \t: %15ld\n", ps.pageReclaims);
  fprintf (where, "  Swaps                      \t: %15ld\n", ps.swaps);
  fprintf (where, "  Block inputs               \t: %15ld\n", ps.inputBlocks);
  fprintf (where, "  Block outputs              \t: %15ld\n", ps.outputBlocks);
  fprintf (where, "  Message inputs             \t: %15ld\n", ps.inputMessages);
  fprintf (where, "  Message outputs            \t: %15ld\n", ps.outputMessages);
  fprintf (where, "  Signals                    \t: %15ld\n", ps.signals);
  fprintf (where, "  Voluntary context switches \t: %15ld\n", ps.voluntaryContextSwitches);
  fprintf (where, "  Forced context switches    \t: %15ld\n", ps.forcedContextSwitches);

}



void startProcessStats (void){
  /* 
   * input   : 
   * returns :
   * output  :
   * desc    :
   */

  struct timezone tz;

  gettimeofday (&tvb, &tz);
  getrusage (RUSAGE_SELF, &rub);

}


void stopProcessStats (void){
  /* 
   * input   : 
   * returns :
   * output  :
   * desc    :
   */

  struct timezone tz;

  gettimeofday (&tve, &tz);
  getrusage (RUSAGE_SELF, &rue);

}
