/* Time-stamp: "sockets.c, Tuesday March 3 1998 22:10:3CST by rjt@ittc.ukans.edu on warped.ittc.ukans.edu" */
/*
 * proj : NetPipe
 * file : sockets.c
 * prog : Roel Jonkman
 * date : november 8th 1995
 * rev  : 3
 * rcs  : $Id$
 * desc : Multiple Socket stuff, not optimized for testing,
 *        optimized for use, still fast though.
 *
 * This software is copyrighted see COPYRIGHTS
 *
 * $Log$
 */


/*
 * defines to force 4.4BSD behaviour
 */

#define _SOCKADDR_LEN

/*
 * generic system includes
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/*
 * system includes
 */

#include <sys/wait.h> 
#include <sys/time.h> /* on a sun resource.h sucks ! */
#include <sys/resource.h>

/*
 * Internet stuff we need to know
 */

#include <netinet/in.h> /* internet stuff */
#include <netdb.h>

/*
 * local includes
 */

#include "sockets.h"
#include "error.h"

/*
 * Generic setup functions TCP/UDP/IP
 */

int createSocket (int *ptr2SocketDescriptor, int addressFamily, int socketType) {
  /* input   : sockettype wanted.
   * returns : NOK if any error OK if none.
   * output  : -
   * desc    : creates a socket.
   */
  
  if ((*ptr2SocketDescriptor = socket (addressFamily, socketType, 0)) < 0) {

    systemError ("socket");
    return NOK;
    
  }

  return OK;
    
}



int bindSocket (int socketDescriptor, int addressFamily, unsigned int *ownAddress, ushort *ownPort) {
  /* input   : 
   * returns : NOK if any error OK if none.
   * output  : 
   * desc    : 
   */

  struct sockaddr_in ownSocket;
  int sizeOf;

  memset ((char*)&ownSocket, 0x0, sizeof(ownSocket));
  
  ownSocket.sin_port        = htons (*ownPort);
  ownSocket.sin_addr.s_addr = htonl (*ownAddress);
  ownSocket.sin_family      = addressFamily;

  if (bind (socketDescriptor,(struct sockaddr *)&ownSocket,
	    sizeof (ownSocket)) < 0) {

    systemError ("bind");
    return NOK;
    
  }

  sizeOf = sizeof(ownSocket);

  if (getsockname (socketDescriptor,(struct sockaddr *)&ownSocket,
	    &sizeOf) < 0) {

    systemError ("getsockname");
    return NOK;
    
  }

  *ownPort = ntohs(ownSocket.sin_port);
  *ownAddress = ntohl (ownSocket.sin_addr.s_addr);

  return OK;

}



int connectSocket (int socketDescriptor, int addressFamily, unsigned int peerAddress, 
		   ushort peerPort) {
  /* input   : 
   * returns : NOK if any error OK if none.
   * output  : -
   * desc    : 
   */

  struct sockaddr_in peerSocket;
  
  memset ((char*)&peerSocket, 0, sizeof ( peerSocket ));

  peerSocket.sin_addr.s_addr = htonl(peerAddress);
  peerSocket.sin_family      = addressFamily;
  peerSocket.sin_port        = htons(peerPort);
  
  if (connect(socketDescriptor, (struct sockaddr*) &peerSocket, sizeof( peerSocket) )
      == SYSTEMERROR) {
    
    systemError ("connect");
    return NOK;
    
  }

  return OK;

}



void shutDownSocket (int socketDescriptor) {
  /* input   : -
   * returns : - 
   * output  : - 
   * desc    : shuts down socket connection
   */

  shutdown (socketDescriptor, 2);

}



void closeSocket (int socketDescriptor) {
  /* input   : -
   * returns : - 
   * output  : - 
   * desc    : closes down socket connection
   */

  close (socketDescriptor);

}



/*
 * TCP specific functions
 */

int setupSocketQueue (int socketDescriptor, int maxPendingRequests) {
  /* input   : maximum number of pending requests
   * returns : NOK if any error OK if none.
   * output  : -
   * desc    : sets up a listen queue, does not block.
   */

  if (listen (socketDescriptor, maxPendingRequests) == SYSTEMERROR) {
    
    systemError ("listen");
    return NOK;
    
  }

  return OK;

}



pid_t wait4Request (int *ptr2SocketDescriptor, int aForkFlag) {
  /* input   : fork, this function forks off a child if fork == TRUE.
   * returns : NOK if any error OK if none.
   * output  : -
   * desc    : waits 4 request of a client on the created socket.
   */

  int aClientSocketDescriptor;
  struct sockaddr peerSocket;
  int length = sizeof (peerSocket);
  pid_t aPid;


  if ((aClientSocketDescriptor = accept (*ptr2SocketDescriptor,
	   &peerSocket, &length)) == SYSTEMERROR) {

    systemError ("accept");
    return NOK;
    
  }
  
  if (!aForkFlag) {
    
    close (*ptr2SocketDescriptor);
    *ptr2SocketDescriptor = aClientSocketDescriptor;
    return OK;
    
  }
  
  if ((aPid = fork()) == 0) {
    
    close (*ptr2SocketDescriptor);
    *ptr2SocketDescriptor = aClientSocketDescriptor;
    
    return CHILD;
    
  } 
  
  if (aPid < 0) {
    
    systemError ("fork");
    return NOK;
    
  }
  
  close (aClientSocketDescriptor);
  return PARENT;
  
}



/*
 * hook stuff
 */

int hookStdin(int socketDescriptor){
  /* input   : -
   * returns : NOK if any error, OK if none.
   * output  : -  
   * desc    : reroutes stdin to the current socket.
   */

  dup2 (socketDescriptor, fileno(stdin));

  if (setvbuf (stdin, (char*)NULL, _IONBF, 0) != 0){

    systemError ("setvbuf");
    return NOK;

  }

  return OK;

}


int hookStdout(int socketDescriptor){
  /* input   : -
   * returns : NOK if any error, OK if none.
   * output  : -  
   * desc    : reroutes stdout to the current socket.
   */

  dup2 (socketDescriptor, fileno(stdout));

  if (setvbuf (stdout, (char*)NULL, _IONBF, 0) != 0){

    systemError ("setvbuf");
    return NOK;

  }

  return OK;

}



FILE *mkFile (int socketDescriptor){
  /* input   : -
   * returns : NOK if any error, OK if none.
   * output  : -  
   * desc    : makes a file struct on a socketdescriptor.
   */
  
  FILE *file;

  if ((file = fdopen (socketDescriptor, "a+")) == NULL) {

    systemError ("fdopen");
    return NULL;

  }

  if (setvbuf (file, (char*)NULL, _IONBF, 0) != 0){

    systemError ("setvbuf");
    return NULL;

  }

  return file;

}



/*
 * Data transfer functions
 */

int readStreamSocket (int socketDescriptor, char *ptr2Data, long *ptr2BytesLeft) {
  /* input   : ptr2Data, and the size of the data to be written
   * returns : OK if no errors NOK if any.
   * output  : -  
   * desc    : reads data from a socket. The first four bytes of the packet are
   *           used to tag the message with a sequencenumber. So those can not be used
   *           for anything else.
   */

  int zeroCounter = 0;
  int bytesRead = 0;

  while (*ptr2BytesLeft > 0) {

    bytesRead = read (socketDescriptor, ptr2Data, *ptr2BytesLeft);

    if (bytesRead <= 0) {

      if (bytesRead < 0) { /* we really got an error */

	systemError ("read");
	return NOK;

      }

      if (bytesRead == 0) { /* maybe peer screwed up here */

	if (zeroCounter >= 10) {

	  error ("read","timed out, broken pipe on a dec alpha running OSF 3.2??");
	  return NOK;

	} else
	  zeroCounter++;

      }

    } else
      zeroCounter = 0; /* consecutive zero's only */

    ptr2Data += bytesRead;   
    *ptr2BytesLeft -= bytesRead;

  }

  return OK;

}



int readMessageSocket (int socketDescriptor, char *ptr2Data, long *ptr2BytesLeft) {
  /* input   : ptr2Data, and the size of the data to be written
   * returns : the number of bytes read
   * output  : -  
   * desc    : reads data from a socket.
   */

  if ((*ptr2BytesLeft = read (socketDescriptor, ptr2Data, *ptr2BytesLeft)) 
      == SYSTEMERROR){

    systemError ("read");
    return NOK;

  }

  return OK;

}



int peekSocket (int socketDescriptor, char *ptr2Data, int *ptr2Length) {
  /* input   : ptr2Data, and the size of the data to be written
   * returns : the number of bytes read
   * output  : -  
   * desc    : peeks at socket data, scary, doesn't work, except for
   *           unconnected sockets maybe.....
   */

  int barf = 0;
  

  
  if ((*ptr2Length = recvfrom (socketDescriptor,  ptr2Data, *ptr2Length,
			   MSG_PEEK, NULL, &barf)) 
      == SYSTEMERROR){

    systemError ("recv");
    return NOK;

  }

  return OK;

}



int writeSocket (int socketDescriptor, char *ptr2Data, long bytesLeft){
  /* input   : ptr2Data, and the size of the data to be written
   * returns : NOK if the size written is not correct OK otherwise
   * output  : -  
   * desc    : writes data to a socket.
   */

  extern int errno;
  int bytesWritten = 0;

  while (bytesLeft > 0) {

    bytesWritten = write (socketDescriptor, ptr2Data, bytesLeft);

    if (bytesWritten == -1) {

      if (errno != ENOBUFS) { /* if ENOBUFS screw it and just continue */

	systemError ("write");
	return NOK;
	
      } else /* ENOBUFS error */
	bytesWritten = 0;

    }
    ptr2Data += bytesWritten;   
    bytesLeft -= bytesWritten;

  }

  return OK;

}



/*
 * miscellaneous support functions
 */

void killZombieWorkers () {
  /* input   : -
   * returns : -
   * output  : -
   * desc    : collects all return code from kids, and kills them effectively that way.
   *           Kids are actually child processes which finished but still wait for their
   *           return codes to be read. This function reads all returncodes available.
   */

   int dummyStatus;
   
   while (waitpid(-1, &dummyStatus, WNOHANG) > 0);

}


int bsdRcvBuf(int socketDescriptor, int anOperation, int *ptr2BufSize) {
  /* input   :
   * returns :
   * output  :
   * desc    :
   */

#ifdef SO_RCVBUF

  int argLength = sizeof (int);
  int status = OK;

  if (anOperation == SET)
    if (setsockopt (socketDescriptor, SOL_SOCKET, SO_RCVBUF, (char*)ptr2BufSize,
 argLength)
                == SYSTEMERROR) {
   
      systemError ("setsockopt");
      status = NOK;

    }

  if (getsockopt (socketDescriptor, SOL_SOCKET, SO_RCVBUF, (char*)ptr2BufSize, &
argLength)
          == SYSTEMERROR) {
   
    systemError ("getsockopt");
    status = NOK;

  }

  return status;

#else

  *ptr2BufSize = 0;

  if (anOperation == SET) {
 
    error ("setsockopt", "Option not available");
    return NOK;

  } else
    return OK;

#endif


}



int bsdXmtBuf (int socketDescriptor, int anOperation, int *ptr2BufSize) {
  /* input   :
   * returns :
   * output  :
   * desc    :
   */

#ifdef SO_SNDBUF

  int argLength = sizeof (int);
  int status = OK;

  if (anOperation == SET)
    if (setsockopt (socketDescriptor, SOL_SOCKET, SO_SNDBUF, (char*)ptr2BufSize,
 argLength)
                == SYSTEMERROR) {

      systemError ("setsockopt");
      status = NOK;

    }

  if (getsockopt (socketDescriptor, SOL_SOCKET, SO_SNDBUF, (char*)ptr2BufSize, &
argLength)
          == SYSTEMERROR) {

    systemError ("getsockopt");
    status = NOK;

  }

  return status;

#else

  *ptr2BufSize = 0;
  if (anOperation == SET) {
 
    error ("setsockopt", "Option not available");
    return NOK;

  } else
    return OK;

#endif

}



int bsdTcpMaxSegmentSize (int socketDescriptor, int anOperation, int *ptr2Size)
{
  /* input   :
   * returns :
   * output  :
   * desc    :
   */

#ifdef TCP_MAXSEG

  int argLength = sizeof (int);
  int status = OK;

  if (anOperation == SET)
    if (setsockopt (socketDescriptor, getprotobyname("tcp")->p_proto, TCP_MAXSEG
,
                                        (char*)ptr2Size, argLength) == SYSTEMERROR) {
   
      systemError ("setsockopt");
      status = NOK;

    }

  if (getsockopt (socketDescriptor, getprotobyname("tcp")->p_proto, TCP_MAXSEG,
                                  (char*)ptr2Size, &argLength) == SYSTEMERROR) {

    systemError ("getsockopt");
    status = NOK;

  }

  return status;

#else

  *ptr2Size = 0;
  if (anOperation == SET) {
 
    error ("setsockopt", "Option not available");
    return NOK;

  } else
    return OK;

#endif

}

