/* Time-stamp: "sockets.h, Thursday October 2 1997 12:6:56CDT by rjonkman@ittc.ukans.edu on pepe.ittc.ukans.edu" */

/*
 * proj : NetPipe
 * file : sockets.h
 * prog : Roel Jonkman
 * date : november 8th 1995
 * rev  : 4
 * rcs  : $Id$
 * desc : Multiple Socket stuff, not optimized for testing, optimized for use.
 *
 * This software is copyrighted see COPYRIGHTS
 *
 * $Log$
 */

#ifndef SOCKETS_H
#define SOCKETS_H

/*
 * defines to force 4.4BSD behaviour
 */

#define _SOCKADDR_LEN

/*
 * we're using some stuff in the function calls
 */

#include <sys/types.h>
#include <sys/socket.h>



/*
 * local includes
 */

#include "ipaddress.h"
#include "gendef.h"



/*
 * defines, above 25, see gendef.h
 */

#define PARENT 26
#define CHILD  27



/*
 * Generic setup functions TCP/UDP/IP
 */

int createSocket (int *socketDescriptor, int addressFamily, int socketType) ;
  /* input   : sockettype wanted.
   * returns : NOK if any error OK if none.
   * output  : -
   * desc    : creates a socket.
   */

int bindSocket (int socketDescriptor, int addressFamily, unsigned int *ownAddress, ushort *ownPort) ;
  /* input   : 
   * returns : NOK if any error OK if none.
   * output  : 
   * desc    : 
   */

int connectSocket (int socketDescriptor, int addressFamily, unsigned int peerAddress, 
		   ushort peerPort) ;
  /* input   : 
   * returns : NOK if any error OK if none.
   * output  : -
   * desc    : 
   */

void shutDownSocket (int socketDescriptor) ;
  /* input   : -
   * returns : - 
   * output  : - 
   * desc    : shuts down socket connection
   */

void closeSocket (int socketDescriptor) ;
  /* input   : -
   * returns : - 
   * output  : - 
   * desc    : closes down socket connection
   */

/*
 * TCP specific functions
 */

int setupSocketQueue (int socketDescriptor, int maxPendingRequests) ;
  /* input   : maximum number of pending requests
   * returns : NOK if any error OK if none.
   * output  : -
   * desc    : sets up a listen queue, does not block.
   */

pid_t wait4Request (int *ptr2SocketDescriptor, int aForkFlag) ;
  /* input   : fork, this function forks off a child if fork == TRUE.
   * returns : NOK if any error OK if none.
   * output  : -
   * desc    : waits 4 request of a client on the created socket.
   */

/*
 * hook stuff
 */

int hookStdin(int socketDescriptor);
  /* input   : -
   * returns : NOK if any error, OK if none.
   * output  : -  
   * desc    : reroutes stdin to the current socket.
   */

int hookStdout(int socketDescriptor);
  /* input   : -
   * returns : NOK if any error, OK if none.
   * output  : -  
   * desc    : reroutes stdout to the current socket.
   */

FILE *mkFile (int socketDescriptor);
  /* input   : -
   * returns : NOK if any error, OK if none.
   * output  : -  
   * desc    : makes a file struct on a socketdescriptor.
   */
  

/*
 * Data transfer functions
 */

int readStreamSocket (int socketDescriptor, char *ptr2Data, long *ptr2BytesLeft) ;
  /* input   : ptr2Data, and the size of the data to be written
   * returns : OK if no errors NOK if any.
   * output  : -  
   * desc    : 
   */

int readMessageSocket (int socketDescriptor, char *ptr2Data, long *ptr2BytesLeft) ;
  /* input   : ptr2Data, and the size of the data to be written
   * returns : the number of bytes read
   * output  : -  
   * desc    : reads data from a socket.
   */

int peekSocket (int socketDescriptor, char *ptr2Data, int *ptr2Length) ;
  /* input   : ptr2Data, and the size of the data to be written
   * returns : the number of bytes read
   * output  : -  
   * desc    : peeks at socket data, scary.
   */

int writeSocket (int socketDescriptor, char *ptr2Data, long bytesLeft);
  /* input   : ptr2Data, and the size of the data to be written
   * returns : NOK if the size written is not correct OK otherwise
   * output  : -  
   * desc    : writes data to a socket.
   */

/*
 * miscellaneous support functions
 */

void killZombieWorkers () ;
  /* input   : -
   * returns : -
   * output  : -
   * desc    : collects all return code from kids, and kills them effectively that way.
   *           Kids are actually child processes which finished but still wait for their
   *           return codes to be read. This function reads all returncodes available.
   */

int bsdRcvBuf(int socketDescriptor, int anOperation, int *ptr2BufSize);
  /* input   :
   * returns :
   * output  :
   * desc    :
   */

int bsdXmtBuf(int socketDescriptor, int anOperation, int *ptr2BufSize);
  /* input   :
   * returns :
   * output  :
   * desc    :
   */

int bsdTcpMaxSegmentSize (int socketDescriptor, int anOperation, int *ptr2Size);
  /* input   :
   * returns :
   * output  :
   * desc    :
   */


#endif











